/*
 
 Copyright (C) 2001-2002       A Nourai
 Copyright (C) 2006            Jacek Piszczek (Mac OSX port)
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 
 See the included (GNU.txt) GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "quakedef.h"
#include "sound.h"
#include <CoreServices/CoreServices.h>
#include <AudioUnit/AudioUnit.h>

// Jacek:
// coreaudio is poorly documented so I'm not 100% sure the code below
// is correct :(

static AudioUnit	gOutputUnit;
static unsigned int readpos;
void *buffer;

OSStatus AudioRender(void                       *inRefCon, 
                     AudioUnitRenderActionFlags *ioActionFlags, 
                     const AudioTimeStamp 		*inTimeStamp, 
                     UInt32 					 inBusNumber, 
                     UInt32 					 inNumberFrames, 
                     AudioBufferList            *ioData)

{

    memcpy(ioData->mBuffers[0].mData,
           buffer + (readpos * 4),
           ioData->mBuffers[0].mDataByteSize);

    if (readpos >= 128 * 1023)
    {
        readpos = 0;
    } else {
        readpos += inNumberFrames;
    }

	return noErr;
}

void SNDDMA_Shutdown()
{
    // stop playback
    AudioOutputUnitStop (gOutputUnit);

    // release the unit
    AudioUnitUninitialize (gOutputUnit);

    // free the unit
	CloseComponent (gOutputUnit);

    // free the buffer memory
    free(buffer);
}

int SNDDMA_GetDMAPos()
{
	shm->samplepos = readpos * 2; 
	return shm->samplepos;
}

void SNDDMA_Submit()
{
}

qboolean SNDDMA_Init()
{
	ComponentResult err = noErr;
    int speed;
    
    shm = &sn;
    
	bzero((void*)shm, sizeof(*shm));    
    
	if (s_khz.value == 44)
		speed = 44100;
	else if (s_khz.value == 22)
		speed = 22050;
	else
		speed = 11025;

	// Open the default output unit
	ComponentDescription desc;
	desc.componentType = kAudioUnitType_Output;
	desc.componentSubType = kAudioUnitSubType_DefaultOutput;
	desc.componentManufacturer = kAudioUnitManufacturer_Apple;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	Component comp = FindNextComponent(NULL, &desc);
	if (comp == NULL) return FALSE;

	err = OpenAComponent(comp, &gOutputUnit);
	if (comp == NULL) return FALSE;
  
	// Set up a callback function to generate output to the output unit
    AURenderCallbackStruct input;
	input.inputProc = AudioRender;
	input.inputProcRefCon = NULL;
    
	err = AudioUnitSetProperty (gOutputUnit, 
								kAudioUnitProperty_SetRenderCallback, 
								kAudioUnitScope_Input,
								0, 
								&input, 
								sizeof(input));
	if (err) 
    {
        CloseComponent(gOutputUnit);
        return FALSE;
    }
  
    // describe our audio data
	AudioStreamBasicDescription streamFormat;
    streamFormat.mSampleRate = speed;
    streamFormat.mFormatID = kAudioFormatLinearPCM;
    streamFormat.mFormatFlags = kLinearPCMFormatFlagIsSignedInteger 
								| kAudioFormatFlagsNativeEndian
								| kLinearPCMFormatFlagIsPacked;
								//| kAudioFormatFlagIsNonInterleaved;
    streamFormat.mBytesPerPacket = 4;	
    streamFormat.mFramesPerPacket = 1;	
    streamFormat.mBytesPerFrame = 4;		
    streamFormat.mChannelsPerFrame = 2;	
    streamFormat.mBitsPerChannel = 16;	

	err = AudioUnitSetProperty (gOutputUnit,
                                kAudioUnitProperty_StreamFormat,
                                kAudioUnitScope_Input,
                                0,
                                &streamFormat,
                                sizeof(AudioStreamBasicDescription));
	if (err) 
    {
        CloseComponent(gOutputUnit);
        return FALSE;
    }
	
    // set the shm structure
    shm->speed = speed;
    shm->samplebits = 16;
    shm->channels = 2;
    shm->samples = 256 * 1024;
    shm->submission_chunk = 1;
    shm->buffer = malloc(128 * 4 * 1025);
    
    // store the buffer ptr in a separate variable, fuhQ may zero the
    // shm->buffer when it feels like it!
    buffer = shm->buffer;
    
    if (shm->buffer == 0)
    {
        CloseComponent(gOutputUnit);
        return FALSE;
    }
            
    // Initialize unit
	err = AudioUnitInitialize(gOutputUnit);
	if (err) 
    {
        CloseComponent(gOutputUnit);
        free(buffer);
        return FALSE;
    }
    
    // start playing :)
    err = AudioOutputUnitStart (gOutputUnit);
	if (err) 
    {
        AudioUnitUninitialize (gOutputUnit);
        CloseComponent(gOutputUnit);
        free(buffer);
        return FALSE;
    }
    
    return TRUE;
}
