/*
 
 Copyright (C) 2001-2002       A Nourai
 Copyright (C) 2006            Jacek Piszczek (Mac OSX port)
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 
 See the included (GNU.txt) GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "quakedef.h"
#include "input.h"
#include "keys.h"
#include "gl_local.h"

#include "vid_macos.h"

#define WARP_WIDTH		320
#define WARP_HEIGHT		200

// note: cocoa code is separated in vid_cocoa.m because of compilation pbs

extern viddef_t vid;

cvar_t _windowed_mouse; // not used but required :)

qboolean vid_hwgamma_enabled = false;

char pal[256*4];

unsigned char *gammatable;

int real_width, real_height;

void VID_Init(unsigned char *palette)
{
	int argnum;
	int i;
	int depth = 24;
    qboolean whpassed = false,dpassed = false;

	vid.width = 1024;
	vid.height = 768;
	vid.maxwarpwidth = WARP_WIDTH;
	vid.maxwarpheight = WARP_HEIGHT;
	vid.numpages = 1;
	vid.colormap = host_colormap;
	vid.fullbright = 256 - LittleLong (*((int *)vid.colormap + 2048));

	argnum = COM_CheckParm("-width");
	if (argnum)
	{
		if (argnum >= com_argc - 1)
			Sys_Error("VID: -width <width>");

		vid.width = Q_atoi(com_argv[argnum+1]);

		if (vid.width == 0)
			Sys_Error("VID: Bad width");
            
        whpassed = true;
	}

	argnum = COM_CheckParm("-height");
	if (argnum)
	{
		if (argnum >= com_argc - 1)
			Sys_Error("VID: -height <height>");

		vid.height = Q_atoi(com_argv[argnum+1]);

		if (vid.height == 0)
			Sys_Error("VID: Bad height");
        
        whpassed = false;
	}

	argnum = COM_CheckParm("-depth");
	if (argnum)
	{
		if (argnum >= com_argc - 1)
			Sys_Error("VID: -depth <depth>");

		depth = Q_atoi(com_argv[argnum+1]);

		if (depth != 15 && depth != 16 && depth != 24)
			Sys_Error("VID: Bad depth");
        
        dpassed = true;
	}

    // initialise the NSApplication and the screen
    initCocoa(whpassed ? vid.width : 0,whpassed ? vid.height : 0,
              dpassed ? depth : 0);

    // initCocoa stores current screen size in vid
	real_width = vid.width;
	real_height = vid.height;


    // calculate the conwidth AFTER the screen has been opened
	if (vid.width <= 640)
    {
		vid.conwidth = vid.width;
		vid.conheight = vid.height;
    }
	else
    {
		vid.conwidth = vid.width/2;
		vid.conheight = vid.height/2;
    }
    
	if ((i = COM_CheckParm("-conwidth")) && i + 1 < com_argc)
    {
		vid.conwidth = Q_atoi(com_argv[i + 1]);
        
		// pick a conheight that matches with correct aspect
		vid.conheight = vid.conwidth * 3 / 4;
    }
    
	vid.conwidth &= 0xfff8; // make it a multiple of eight
    
	if ((i = COM_CheckParm("-conheight")) && i + 1 < com_argc)
		vid.conheight = Q_atoi(com_argv[i + 1]);
    
	if (vid.conwidth < 320)
		vid.conwidth = 320;
    
	if (vid.conheight < 200)
		vid.conheight = 200;
    
	vid.rowbytes = vid.width;
	vid.direct = 0; /* Isn't used anywhere, but whatever. */
	vid.aspect = ((float)vid.height / (float)vid.width) * (320.0 / 240.0);

	if (vid.conheight > vid.height)
		vid.conheight = vid.height;
	if (vid.conwidth > vid.width)
		vid.conwidth = vid.width;

	vid.width = vid.conwidth;
	vid.height = vid.conheight;

	GL_Init();

	Check_Gamma(palette);

	VID_SetPalette(palette);

	vid.recalc_refdef = 1;
}

void VID_Shutdown()
{
    killCocoa();
}

void VID_ShiftPalette(unsigned char *p)
{
	VID_SetPalette(p);
}

void Sys_SendKeyEvents()
{
}

void VID_LockBuffer()
{
}

void VID_UnlockBuffer()
{
}

qboolean VID_IsLocked()
{
	return 0;
}

void D_BeginDirectRect(int x, int y, byte *pbitmap, int width, int height)
{
}

void D_EndDirectRect(int x, int y, int width, int height)
{
}

void VID_SetCaption(char *text)
{
}

void GL_BeginRendering (int *x, int *y, int *width, int *height)
{
	*x = *y = 0;
	*width = real_width;
	*height = real_height;
}

void GL_EndRendering(void)
{
    flushCocoa();
}

void VID_SetDeviceGammaRamp(unsigned short *ramps)
{
    cocoaGamma(ramps,ramps+256,ramps+512);
}
